@@ -88,8 +88,6 @@ use IO::File;
 
 $diag = 1, shift @ARGV if $ARGV[0] eq "-d";
 
-($out_format) = @ARGV;
-
 $| = 1;
 
 $tstruct_previtem = $node = "Top";
@@ -103,8 +101,7 @@ print "Reading input...";
 $pname = "para000000";
 @pnames = @pflags = ();
 $para = undef;
-while (defined($_ = <STDIN>)) {
-  $_ = &untabify($_);
+while (<>) {
   &check_include($_);
 }
 &got_para($para);
@@ -128,50 +125,22 @@ if ($diag) {
 }
 
 # OK. Write out the various output files.
-if ($out_format eq 'txt') {
-    print "Producing text output: ";
-    &write_txt;
-    print "done.\n";
-} elsif ($out_format eq 'html') {
-    print "Producing HTML output: ";
-    &write_html;
-    print "done.\n";
-} elsif ($out_format eq 'texi') {
-    print "Producing Texinfo output: ";
-    &write_texi;
-    print "done.\n";
-} elsif ($out_format eq 'hlp') {
-    print "Producing WinHelp output: ";
-    &write_hlp;
-    print "done.\n";
-} elsif ($out_format eq 'dip') {
-    print "Producing Documentation Intermediate Paragraphs: ";
-    &write_dip;
-    print "done.\n";
-} else {
-    die "$0: unknown output format: $out_format\n";
-}
+print "Producing text output: ";
+&write_txt;
+print "done.\n";
+print "Producing HTML output: ";
+&write_html;
+print "done.\n";
+print "Producing Texinfo output: ";
+&write_texi;
+print "done.\n";
+print "Producing WinHelp output: ";
+&write_hlp;
+print "done.\n";
+print "Producing Documentation Intermediate Paragraphs: ";
+&write_dip;
+print "done.\n";
 
-sub untabify($) {
-  my($s) = @_;
-  my $o = '';
-  my($c, $i, $p);
-
-  $p = 0;
-  for ($i = 0; $i < length($s); $i++) {
-    $c = substr($s, $i, 1);
-    if ($c eq "\t") {
-      do {
-	$o .= ' ';
-	$p++;
-      } while ($p & 7);
-    } else {
-      $o .= $c;
-      $p++;
-    }
-  }
-  return $o;
-}
 sub check_include {
   local $_ = shift;
   if (/\\& (\S+)/) {
@@ -1311,12 +1280,11 @@ sub write_hlp {
       # Code paragraph.
       print "\\keep\\f1\\sb120\n";
       foreach $i (@$pname) {
-	my $x = $i;
         warn "code line longer than 68 chars: $i\n" if length $i > 68;
-	$x =~ s/\\/\\\\/g;
-	$x =~ s/\{/\\\{/g;
-	$x =~ s/\}/\\\}/g;
-        print "$x\\par\\sb0\n";
+	$i =~ s/\\/\\\\/g;
+	$i =~ s/\{/\\\{/g;
+	$i =~ s/\}/\\\}/g;
+        print "$i\\par\\sb0\n";
       }
       $newpar = "\\pard\\f0\\sb120\n";
     } elsif ($ptype eq "bull" || $ptype eq "norm") {
