@@ -2,7 +2,7 @@
  *
  * The Netwide Assembler is copyright (C) 1996 Simon Tatham and
  * Julian Hall. All rights reserved. The software is
- * redistributable under the licence given in the file "Licence"
+ * redistributable under the license given in the file "LICENSE"
  * distributed in the NASM archive.
  */
 
@@ -15,6 +15,7 @@
 #include <ctype.h>
 #include <inttypes.h>
 #include <limits.h>
+#include <time.h>
 
 #include "nasm.h"
 #include "nasmlib.h"
@@ -43,16 +44,18 @@ static void report_error_gnu(int severity, const char *fmt, ...);
 static void report_error_vc(int severity, const char *fmt, ...);
 static void report_error_common(int severity, const char *fmt,
                                 va_list args);
-static int is_suppressed_warning(int severity);
+static bool is_suppressed_warning(int severity);
 static void usage(void);
 static efunc report_error;
 
 static int using_debug_info, opt_verbose_info;
 bool tasm_compatible_mode = false;
-int pass0;
+int pass0, passn;
 int maxbits = 0;
 int globalrel = 0;
 
+time_t official_compile_time;
+
 static char inname[FILENAME_MAX];
 static char outname[FILENAME_MAX];
 static char listname[FILENAME_MAX];
@@ -160,8 +163,86 @@ static void nasm_fputs(const char *line, FILE * outfile)
         puts(line);
 }
 
+/* Convert a struct tm to a POSIX-style time constant */
+static int64_t posix_mktime(struct tm *tm)
+{
+    int64_t t;
+    int64_t y = tm->tm_year;
+
+    /* See IEEE 1003.1:2004, section 4.14 */
+
+    t = (y-70)*365 + (y-69)/4 - (y-1)/100 + (y+299)/400;
+    t += tm->tm_yday;
+    t *= 24;
+    t += tm->tm_hour;
+    t *= 60;
+    t += tm->tm_min;
+    t *= 60;
+    t += tm->tm_sec;
+
+    return t;
+}
+
+static void define_macros_early(void)
+{
+    char temp[128];
+    struct tm lt, *lt_p, gm, *gm_p;
+    int64_t posix_time;
+
+    lt_p = localtime(&official_compile_time);
+    if (lt_p) {
+	lt = *lt_p;
+
+	strftime(temp, sizeof temp, "__DATE__=\"%Y-%m-%d\"", &lt);
+	pp_pre_define(temp);
+	strftime(temp, sizeof temp, "__DATE_NUM__=%Y%m%d", &lt);
+	pp_pre_define(temp);
+	strftime(temp, sizeof temp, "__TIME__=\"%H:%M:%S\"", &lt);
+	pp_pre_define(temp);
+	strftime(temp, sizeof temp, "__TIME_NUM__=%H%M%S", &lt);
+	pp_pre_define(temp);
+    }
+
+    gm_p = gmtime(&official_compile_time);
+    if (gm_p) {
+	gm = *gm_p;
+
+	strftime(temp, sizeof temp, "__UTC_DATE__=\"%Y-%m-%d\"", &gm);
+	pp_pre_define(temp);
+	strftime(temp, sizeof temp, "__UTC_DATE_NUM__=%Y%m%d", &gm);
+	pp_pre_define(temp);
+	strftime(temp, sizeof temp, "__UTC_TIME__=\"%H:%M:%S\"", &gm);
+	pp_pre_define(temp);
+	strftime(temp, sizeof temp, "__UTC_TIME_NUM__=%H%M%S", &gm);
+	pp_pre_define(temp);
+    }
+    
+    if (gm_p)
+	posix_time = posix_mktime(&gm);
+    else if (lt_p)
+	posix_time = posix_mktime(&lt);
+    else
+	posix_time = 0;
+
+    if (posix_time) {
+	snprintf(temp, sizeof temp, "__POSIX_TIME__=%"PRId64, posix_time);
+	pp_pre_define(temp);
+    }
+}
+
+static void define_macros_late(void)
+{
+    char temp[128];
+
+    snprintf(temp, sizeof(temp), "__OUTPUT_FORMAT__=%s\n",
+	     ofmt->shortname);
+    pp_pre_define(temp);
+}
+
 int main(int argc, char **argv)
 {
+    time(&official_compile_time);
+
     pass0 = 1;
     want_usage = terminate_after_phase = false;
     report_error = report_error_gnu;
@@ -179,6 +260,10 @@ int main(int argc, char **argv)
 
     register_output_formats();
 
+    /* Define some macros dependent on the runtime, but not
+       on the command line. */
+    define_macros_early();
+
     parse_cmdline(argc, argv);
 
     if (terminate_after_phase) {
@@ -197,12 +282,7 @@ int main(int argc, char **argv)
     eval_global_info(ofmt, lookup_label, &location);
 
     /* define some macros dependent of command-line */
-    {
-        char temp[64];
-        snprintf(temp, sizeof(temp), "__OUTPUT_FORMAT__=%s\n",
-                 ofmt->shortname);
-        pp_pre_define(temp);
-    }
+    define_macros_late();
 
     switch (operating_mode) {
     case op_depend_missing_ok:
@@ -437,12 +517,8 @@ static bool process_arg(char *p, char *q)
 			/* -O1 -> optimizing == 0, 0.98.09 behaviour */
 			if (opt < 2)
 			    optimizing = opt - 1;
-			else if (opt <= 5)
-			    /* The optimizer seems to have problems with
-			       < 5 passes?  Hidden bug? */
-			    optimizing = 5;	/* 5 passes */
 			else
-			    optimizing = opt;   /* More than 5 passes */
+			    optimizing = opt;
 			break;
 			
 		    case 'v':
@@ -795,6 +871,20 @@ static void process_args(char *args)
         process_arg(arg, NULL);
 }
 
+static void process_response_file(const char *file)
+{
+    char str[2048];
+    FILE *f = fopen(file, "r");
+    if (!f) {
+	perror(file);
+	exit(-1);
+    }
+    while (fgets(str, sizeof str, f)) {
+	process_args(str);
+    }
+    fclose(f);
+}
+
 static void parse_cmdline(int argc, char **argv)
 {
     FILE *rfile;
@@ -826,19 +916,7 @@ static void parse_cmdline(int argc, char **argv)
              * different to the -@resp file processing below for regular
              * NASM.
              */
-            char *str = malloc(2048);
-            FILE *f = fopen(&argv[0][1], "r");
-            if (!str) {
-                printf("out of memory");
-                exit(-1);
-            }
-            if (f) {
-                while (fgets(str, 2048, f)) {
-                    process_args(str);
-                }
-                fclose(f);
-            }
-            free(str);
+	    process_response_file(argv[0]+1);
             argc--;
             argv++;
         }
@@ -901,8 +979,7 @@ static void assemble_file(char *fname)
     int64_t offs;
     struct tokenval tokval;
     expr *e;
-    int pass, pass_max;
-    int pass_cnt = 0;           /* count actual passes */
+    int pass_max;
 
     if (cmd_sb == 32 && cmd_cpu < IF_386)
         report_error(ERR_FATAL, "command line: "
@@ -910,15 +987,15 @@ static void assemble_file(char *fname)
 
     pass_max = (optimizing > 0 ? optimizing : 0) + 2;   /* passes 1, optimizing, then 2 */
     pass0 = !(optimizing > 0);  /* start at 1 if not optimizing */
-    for (pass = 1; pass <= pass_max && pass0 <= 2; pass++) {
+    for (passn = 1; pass0 <= 2; passn++) {
         int pass1, pass2;
         ldfunc def_label;
 
-        pass1 = pass < pass_max ? 1 : 2;        /* seq is 1, 1, 1,..., 1, 2 */
-        pass2 = pass > 1 ? 2 : 1;       /* seq is 1, 2, 2,..., 2, 2 */
-        /*      pass0                            seq is 0, 0, 0,..., 1, 2 */
+        pass1 = pass0 == 2 ? 2 : 1;	/* 1, 1, 1, ..., 1, 2 */
+        pass2 = passn > 1  ? 2 : 1;     /* 1, 2, 2, ..., 2, 2 */
+        /* pass0                           0, 0, 0, ..., 1, 2 */
 
-        def_label = pass > 1 ? redefine_label : define_label;
+        def_label = passn > 1 ? redefine_label : define_label;
 
         globalbits = sb = cmd_sb;   /* set 'bits' to command line default */
         cpu = cmd_cpu;
@@ -930,7 +1007,7 @@ static void assemble_file(char *fname)
         global_offset_changed = false;  /* set by redefine_label */
         location.segment = ofmt->section(NULL, pass2, &sb);
         globalbits = sb;
-        if (pass > 1) {
+        if (passn > 1) {
             saa_rewind(forwrefs);
             forwref = saa_rstruct(forwrefs);
             raa_free(offsets);
@@ -938,7 +1015,7 @@ static void assemble_file(char *fname)
         }
         preproc->reset(fname, pass1, report_error, evaluate, &nasmlist);
         globallineno = 0;
-        if (pass == 1)
+        if (passn == 1)
             location.known = true;
         location.offset = offs = GET_CURR_OFFS;
 
@@ -977,7 +1054,7 @@ static void assemble_file(char *fname)
                             *q++ = '\0';
                             ofmt->symdef(value, 0L, 0L, 3, q);
                         }
-                    } else if (pass == 1) {     /* pass == 1 */
+                    } else if (passn == 1) {
                         q = value;
                         validid = true;
                         if (!isidstart(*q))
@@ -1117,7 +1194,7 @@ static void assemble_file(char *fname)
                             abs_seg = reloc_seg(e);
                             abs_offset = reloc_value(e);
                         }
-                    } else if (pass == 1)
+                    } else if (passn == 1)
                         abs_offset = 0x100;     /* don't go near zero in case of / */
                     else
                         report_error(ERR_PANIC, "invalid ABSOLUTE address "
@@ -1138,13 +1215,13 @@ static void assemble_file(char *fname)
                     }
                     *q++ = 0;
                     if (!validid) {
-                        report_error(pass == 1 ? ERR_NONFATAL : ERR_PANIC,
+                        report_error(passn == 1 ? ERR_NONFATAL : ERR_PANIC,
                                      "identifier expected after DEBUG");
                         break;
                     }
                     while (*p && isspace(*p))
                         p++;
-                    if (pass == pass_max)
+                    if (pass0 == 2)
                         ofmt->current_dfmt->debug_directive(debugid, p);
                     break;
                 case D_WARNING:		/* [WARNING {+|-}warn-name] */
@@ -1228,7 +1305,7 @@ static void assemble_file(char *fname)
                 parse_line(pass1, line, &output_ins,
                            report_error, evaluate, def_label);
 
-                if (!(optimizing > 0) && pass == 2) {
+                if (!(optimizing > 0) && pass0 == 2) {
                     if (forwref != NULL && globallineno == forwref->lineno) {
                         output_ins.forw_ref = true;
                         do {
@@ -1242,7 +1319,7 @@ static void assemble_file(char *fname)
                 }
 
                 if (!(optimizing > 0) && output_ins.forw_ref) {
-                    if (pass == 1) {
+                    if (passn == 1) {
                         for (i = 0; i < output_ins.operands; i++) {
                             if (output_ins.oprs[i].
                                 opflags & OPFLAG_FORWARD) {
@@ -1253,7 +1330,7 @@ static void assemble_file(char *fname)
                                 fwinf->operand = i;
                             }
                         }
-                    } else {    /* pass == 2 */
+                    } else {    /* passn > 1 */
                         /*
                          * Hack to prevent phase error in the code
                          *   rol ax,x
@@ -1281,7 +1358,7 @@ static void assemble_file(char *fname)
 			    output_ins.oprs[1].type &= ~REG_SMASK;
                         }
 
-                    }           /* pass == 2 */
+                    }
 
                 }
 
@@ -1334,7 +1411,7 @@ static void assemble_file(char *fname)
                                 report_error(ERR_NONFATAL,
                                              "bad syntax for EQU");
                         }
-                    } else {    /* pass == 2 */
+                    } else {
                         /*
                          * Special `..' EQUs get processed here, except
                          * `..@' macro processor EQUs which are done above.
@@ -1369,7 +1446,7 @@ static void assemble_file(char *fname)
                                 report_error(ERR_NONFATAL,
                                              "bad syntax for EQU");
                         }
-                    }           /* pass == 2 */
+                    }
                 } else {        /* instruction isn't an EQU */
 
                     if (pass1 == 1) {
@@ -1447,7 +1524,7 @@ static void assemble_file(char *fname)
                          * flagged as an error on pass 2
                          */
 
-                    } else {    /* pass == 2 */
+                    } else {
                         offs += assemble(location.segment, offs, sb, cpu,
                                          &output_ins, ofmt, report_error,
                                          &nasmlist);
@@ -1475,22 +1552,17 @@ static void assemble_file(char *fname)
                 usage();
             exit(1);
         }
-        pass_cnt++;
-        if (pass > 1 && !global_offset_changed) {
+        if (passn >= pass_max - 2 ||
+	    (passn > 1 && !global_offset_changed))
             pass0++;
-            if (pass0 == 2)
-                pass = pass_max - 1;
-        } else if (!(optimizing > 0))
-            pass0++;
-
-    }                           /* for (pass=1; pass<=2; pass++) */
+    }
 
     preproc->cleanup(0);
     nasmlist.cleanup();
 #if 1
     if (optimizing > 0 && opt_verbose_info)     /*  -On and -Ov switches */
         fprintf(stdout,
-                "info:: assembly required 1+%d+1 passes\n", pass_cnt - 2);
+                "info:: assembly required 1+%d+1 passes\n", passn-3);
 #endif
 }                               /* exit from assemble_file (...) */
 
@@ -1622,18 +1694,16 @@ static void report_error_vc(int severity, const char *fmt, ...)
  * @param severity the severity of the warning or error
  * @return true if we should abort error/warning printing
  */
-static int is_suppressed_warning(int severity)
+static bool is_suppressed_warning(int severity)
 {
     /*
      * See if it's a suppressed warning.
      */
-    return ((severity & ERR_MASK) == ERR_WARNING &&
-            (severity & ERR_WARN_MASK) != 0 &&
-            suppressed[(severity & ERR_WARN_MASK) >> ERR_WARN_SHR]) ||
-        /*
-         * See if it's a pass-one only warning and we're not in pass one.
-         */
-        ((severity & ERR_PASS1) && pass0 == 2);
+    return (severity & ERR_MASK) == ERR_WARNING &&
+	(((severity & ERR_WARN_MASK) != 0 &&
+	  suppressed[(severity & ERR_WARN_MASK) >> ERR_WARN_SHR]) ||
+	 /* See if it's a pass-one only warning and we're not in pass one. */
+	 ((severity & ERR_PASS1) && pass0 != 1));
 }
 
 /**
